<?php

namespace Bixo\Product\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Product\Forms\Description as DescriptionForm;
use Bixo\Product\Http\Requests\DescriptionResourceRequest;
use Bixo\Product\Http\Resources\DescriptionResource;
use Bixo\Product\Http\Resources\DescriptionsCollection;
use Bixo\Product\Models\Description;
use Bixo\Product\Actions\DescriptionAction;
use Bixo\Product\Actions\DescriptionActions;


/**
 * Resource controller class for description.
 */
class DescriptionResourceController extends BaseController
{

    /**
     * Initialize description resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = DescriptionForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.product.modules'), 'product', guard_url('product'));
            return $next($request);
        });
    }

    /**
     * Display a list of description.
     *
     * @return Response
     */
    public function index(DescriptionResourceRequest $request)
    {
        $request = $request->all();
        $page = DescriptionActions::run('paginate', $request);

        $data = new DescriptionsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('product::description.names'))
            ->view('product::description.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display description.
     *
     * @param Request $request
     * @param Model   $description
     *
     * @return Response
     */
    public function show(DescriptionResourceRequest $request, Description $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new DescriptionResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('product::description.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('product::description.show')
            ->output();
    }

    /**
     * Show the form for creating a new description.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DescriptionResourceRequest $request, Description $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new DescriptionResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('product::description.name'))
            ->view('product::description.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new description.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DescriptionResourceRequest $request, Description $model)
    {
        try {
            $request = $request->all();
            $model = DescriptionAction::run('store', $model, $request);
            $data = new DescriptionResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('product::description.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('product/description/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/product/description'))
                ->redirect();
        }

    }

    /**
     * Show description for editing.
     *
     * @param Request $request
     * @param Model   $description
     *
     * @return Response
     */
    public function edit(DescriptionResourceRequest $request, Description $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new DescriptionResource($model);
        // return view('product::description.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('product::description.name'))
            ->view('product::description.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the description.
     *
     * @param Request $request
     * @param Model   $description
     *
     * @return Response
     */
    public function update(DescriptionResourceRequest $request, Description $model)
    {
        try {
            $request = $request->all();
            $model = DescriptionAction::run('update', $model, $request);
            $data = new DescriptionResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('product::description.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('product/description/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('product/description/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the description.
     *
     * @param Model   $description
     *
     * @return Response
     */
    public function destroy(DescriptionResourceRequest $request, Description $model)
    {
        try {

            $request = $request->all();
            $model = DescriptionAction::run('destroy', $model, $request);
            $data = new DescriptionResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('product::description.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('product/description/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('product/description/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
