<?php

namespace Bixo\Product\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Product\Forms\Portal as PortalForm;
use Bixo\Product\Http\Requests\PortalResourceRequest;
use Bixo\Product\Http\Resources\PortalResource;
use Bixo\Product\Http\Resources\PortalsCollection;
use Bixo\Product\Models\Portal;
use Bixo\Product\Actions\PortalAction;
use Bixo\Product\Actions\PortalActions;


/**
 * Resource controller class for portal.
 */
class PortalResourceController extends BaseController
{

    /**
     * Initialize portal resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = PortalForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.product.modules'), 'product', guard_url('product'));
            return $next($request);
        });
    }

    /**
     * Display a list of portal.
     *
     * @return Response
     */
    public function index(PortalResourceRequest $request)
    {
        $request = $request->all();
        $page = PortalActions::run('paginate', $request);

        $data = new PortalsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('product::portal.names'))
            ->view('product::portal.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display portal.
     *
     * @param Request $request
     * @param Model   $portal
     *
     * @return Response
     */
    public function show(PortalResourceRequest $request, Portal $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new PortalResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('product::portal.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('product::portal.show')
            ->output();
    }

    /**
     * Show the form for creating a new portal.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PortalResourceRequest $request, Portal $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new PortalResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('product::portal.name'))
            ->view('product::portal.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new portal.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PortalResourceRequest $request, Portal $model)
    {
        try {
            $request = $request->all();
            $model = PortalAction::run('store', $model, $request);
            $data = new PortalResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('product::portal.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('product/portal/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/product/portal'))
                ->redirect();
        }

    }

    /**
     * Show portal for editing.
     *
     * @param Request $request
     * @param Model   $portal
     *
     * @return Response
     */
    public function edit(PortalResourceRequest $request, Portal $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new PortalResource($model);
        // return view('product::portal.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('product::portal.name'))
            ->view('product::portal.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the portal.
     *
     * @param Request $request
     * @param Model   $portal
     *
     * @return Response
     */
    public function update(PortalResourceRequest $request, Portal $model)
    {
        try {
            $request = $request->all();
            $model = PortalAction::run('update', $model, $request);
            $data = new PortalResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('product::portal.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('product/portal/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('product/portal/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the portal.
     *
     * @param Model   $portal
     *
     * @return Response
     */
    public function destroy(PortalResourceRequest $request, Portal $model)
    {
        try {

            $request = $request->all();
            $model = PortalAction::run('destroy', $model, $request);
            $data = new PortalResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('product::portal.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('product/portal/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('product/portal/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
