<?php

namespace Bixo\Product\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Product\Forms\Property as PropertyForm;
use Bixo\Product\Http\Requests\PropertyResourceRequest;
use Bixo\Product\Http\Resources\PropertyResource;
use Bixo\Product\Http\Resources\PropertiesCollection;
use Bixo\Product\Models\Property;
use Bixo\Product\Actions\PropertyAction;
use Bixo\Product\Actions\PropertyActions;


/**
 * Resource controller class for property.
 */
class PropertyResourceController extends BaseController
{

    /**
     * Initialize property resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = PropertyForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.product.modules'), 'product', guard_url('product'));
            return $next($request);
        });
    }

    /**
     * Display a list of property.
     *
     * @return Response
     */
    public function index(PropertyResourceRequest $request)
    {
        $request = $request->all();
        $page = PropertyActions::run('paginate', $request);

        $data = new PropertiesCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('product::property.names'))
            ->view('product::property.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display property.
     *
     * @param Request $request
     * @param Model   $property
     *
     * @return Response
     */
    public function show(PropertyResourceRequest $request, Property $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new PropertyResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('product::property.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('product::property.show')
            ->output();
    }

    /**
     * Show the form for creating a new property.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PropertyResourceRequest $request, Property $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new PropertyResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('product::property.name'))
            ->view('product::property.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new property.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PropertyResourceRequest $request, Property $model)
    {
        try {
            $request = $request->all();
            $model = PropertyAction::run('store', $model, $request);
            $data = new PropertyResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('product::property.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('product/property/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/product/property'))
                ->redirect();
        }

    }

    /**
     * Show property for editing.
     *
     * @param Request $request
     * @param Model   $property
     *
     * @return Response
     */
    public function edit(PropertyResourceRequest $request, Property $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new PropertyResource($model);
        // return view('product::property.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('product::property.name'))
            ->view('product::property.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the property.
     *
     * @param Request $request
     * @param Model   $property
     *
     * @return Response
     */
    public function update(PropertyResourceRequest $request, Property $model)
    {
        try {
            $request = $request->all();
            $model = PropertyAction::run('update', $model, $request);
            $data = new PropertyResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('product::property.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('product/property/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('product/property/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the property.
     *
     * @param Model   $property
     *
     * @return Response
     */
    public function destroy(PropertyResourceRequest $request, Property $model)
    {
        try {

            $request = $request->all();
            $model = PropertyAction::run('destroy', $model, $request);
            $data = new PropertyResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('product::property.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('product/property/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('product/property/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
